#!/usr/bin/env python3

from pypol import *

a, x, y, z = symbols('a x y z')

sq1 = Le(0, x) & Le(x, 2) & Le(0, y) & Le(y, 2)
sq2 = Le(2, x) & Le(x, 4) & Le(2, y) & Le(y, 4)
sq3 = Le(0, x) & Le(x, 3) & Le(0, y) & Le(y, 3)
sq4 = Le(1, x) & Le(x, 2) & Le(1, y) & Le(y, 2)
sq5 = Le(1, x) & Le(x, 2) & Le(1, y)
sq6 = Le(1, x) & Le(x, 2) & Le(1, y) & Eq(y, 3)
sq7 = Le(0, x) & Le(x, 2) & Le(0, y) & Eq(z, 2) & Le(a, 3)
universe = Polyhedron([])
q = sq1 - sq2
e = Empty

print('sq1 =', sq1) #print correct square
print('sq2 =', sq2) #print correct square
print('sq3 =', sq3) #print correct square
print('sq4 =', sq4) #print correct square
print('universe =', universe) #print correct square
print()
print('¬sq1 =', ~sq1) #test complement
print()
print('sq1 + sq1 =', sq1 + sq2) #test addition
print('sq1 + sq2 =', Polyhedron(sq1 + sq2)) #test addition
print()
print('universe + universe =', universe + universe)#test addition
print('universe - universe =', universe - universe) #test subtraction
print()
print('sq2 - sq1 =', sq2 - sq1) #test subtraction
print('sq2 - sq1 =', Polyhedron(sq2 - sq1)) #test subtraction
print('sq1 - sq1 =', Polyhedron(sq1 - sq1)) #test subtraction
print()
print('sq1 ∩ sq2 =', sq1 & sq2) #test intersection
print('sq1 ∪ sq2 =', sq1 | sq2) #test union
print()
print('sq1 ⊔ sq2 =', Polyhedron(sq1 | sq2)) # test convex union
print()
print('check if sq1 and sq2 disjoint:', sq1.isdisjoint(sq2)) #should return false
print()
print('sq1 disjoint:', sq1.disjoint()) #make disjoint
print('sq2 disjoint:', sq2.disjoint()) #make disjoint
print()
print('is square 1 universe?:', sq1.isuniverse()) #test if square is universe
print('is u universe?:', universe.isuniverse()) #test if square is universe
print()
print('is sq1 a subset of sq2?:', sq1.issubset(sq2)) #test issubset()
print('is sq4 less than sq3?:', sq4.__lt__(sq3)) # test lt(), must be a strict subset
print()
print('lexographic min of sq1:', sq1.lexmin()) #test lexmin()
print('lexographic max of sq1:', sq1.lexmax()) #test lexmin()
print()
print('lexographic min of sq2:', sq2.lexmin()) #test lexmax()
print('lexographic max of sq2:', sq2.lexmax()) #test lexmax()
print()
print('Polyhedral hull of sq1 + sq2 is:', q.aspolyhedron()) #test polyhedral hull
print()
print('is sq1 bounded?', sq1.isbounded()) #unbounded should return True
print('is sq5 bounded?', sq5.isbounded()) #unbounded should return False
print()
print('sq6:', sq6)
print('sq6 simplified:', sq6.sample())
print()
print(universe.project([x]))
print('sq7 with out constraints involving y and a', sq7.project([a, z, x, y])) #drops dims that are passed
print()
print('sq1 has {} parameters'.format(sq1.num_parameters()))
print()
print('does sq1 constraints involve x?', sq1.involves_dims([x]))
